#ifndef _SYSTEM_CONFIG_H_
#define _SYSTEM_CONFIG_H_

#include "w3300.h"      // basic types

#include <vector>

// forward class references
class SerialCfgState;

// =============================================================
// system configuration class
// =============================================================
// The idea is to encapsulate some of the gross system configuration
// state into a central place.  the main benefit is that when changing
// configuration, there are two sets of configuration: the current one
// and the one being twiddled.  this makes it easy to do.

class SystemCfg
{
public:
    SystemCfg();   // gets config from .ini file
    ~SystemCfg();

    SystemCfg(const SystemCfg &obj);             // copy
    SystemCfg &operator=(const SystemCfg &rhs);  // assignment

    void Load();          // read from configuration file
    void Save() const;    // save to configuration file

    bool operator==(const SystemCfg &rhs) const;
    bool operator!=(const SystemCfg &rhs) const;

    bool GetCpuSpeedRegulated() const { return m_cpu_speed_regulated; }
    int  GetRamKB()             const { return m_ram_kb; }
    int  GetNumTerminals()      const { return m_num_terminals; }
    int  GetNumCassetteDrives() const { return m_num_cassette_drives; }
    SerialCfgState *GetSerialCfgState(int n) const
                                      { return m_serial[n]; }

    void SetCpuSpeedRegulated(bool v);
    void SetRamKB(int v);
    void SetNumTerminals(int v);
    void SetNumCassetteDrives(int v);
    void SetSerialCfg(int n, SerialCfgState &cfg);

private:
    bool m_cpu_speed_regulated;
    int  m_ram_kb;
    int  m_num_terminals;
    int  m_num_cassette_drives;

    std::vector <SerialCfgState *> m_serial;
};

#endif // _SYSTEM_CONFIG_H_
