// emulate a Wang 3315 terminal, which is a rebadged ASR33 TTY.
//

#ifndef _TERM3315_H_
#define _TERM3315_H_

#include "IoDevice.h"

#if 1 // using serial port
#include "ctb-0.16/iobase.h"
#include "ctb-0.16/serport.h"
#include "ctb-0.16/timer.h"
#endif

// forward class references
class Term3315_kbd;
class Term3315_prt;
class Term3315_pun;
class Cpu3300;
class Scheduler;
class SerialCfgState;
class UiTerm3315;

// ======================================================================
// This is mostly a shell that proxies methods to pimpl subcomponents:
//    Term3315_kbd, Term3315_prt, Term3315_pun
//    (keyboard)    (printer)     (reader/punch)
// ======================================================================

class Term3315: public IoDevice
{
public:
    // create terminal #"termnum" and attach it to the CPU
    Term3315(Cpu3300 *cpu, Scheduler &Scheduler, int termnum);

    // clean up
    ~Term3315();

    // mechanism for UI to notify us of keystroke to our terminal
    void Keystroke(uint8 keycode);

    // the idle loop calls this to give the terms a chance to check
    // the serial ports for activity
    void PollCheck();

    // overridden IoDevice methods
    void Reset();
    void Clear();
    void WrControl(int addr, int data);
    int  RdStatus(int addr) const;
    void WrData(int addr, int data);
    int  RdData(int addr) const;
    void IAck(int addr);

    // paper tape file attributes
    bool  TapeMounted(const unit_t unit) const;
    void  MountTape(const unit_t unit, const char *filename);
    void  UnmountTape(const unit_t unit);
    char *GetTapePath(const unit_t unit) const;
    int   TapePosition(const unit_t unit);
    void  SetTapePosition(const unit_t unit, int position);

    // paper tape reader/punch enable switches
    void ReaderStart();
    void ReaderStop();
    bool ReaderEnabled() const;
    bool PunchEnabled() const;
    void SetPunchEnabled(bool enable);

    bool setTtyConfiguration(SerialCfgState *cfg);
    SerialCfgState *getTtyConfiguration() const;

private:
    UiTerm3315   *m_ui;                 // user interface routines
    Term3315_kbd *m_kbd;                // keyboard
    Term3315_prt *m_prt;                // printer
    Term3315_pun *m_pun;                // punch

    // serial port configuration (NULL=no associated serial port)
    ctb::SerialPort *m_serport;
    SerialCfgState  *m_scs;

    bool m_initialized;        // true after construction is done
    bool Initialized() const { return m_initialized; }
};

#endif // _TERM3315_H_
