// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

#include "system3300.h"
#include "UiSystem.h"
#include "Host.h"
#include "UiSerialCfgDlg.h"

#include <algorithm>

enum
{
    ID_TTY_ENABLE = 1,
    ID_TTY_PORT,
    ID_TTY_BAUD,
    ID_TTY_PARITY,
    ID_TTY_BITS,
    ID_TTY_STOPBITS,
    ID_TTY_FLOWCTL,

    ID_BTN_REVERT = 300
};


// dialog events to catch
BEGIN_EVENT_TABLE(SerialCfgDlg, wxDialog)
    EVT_CHECKBOX(ID_TTY_ENABLE,   SerialCfgDlg::OnTtyEnable)
    EVT_RADIOBOX(ID_TTY_PORT,     SerialCfgDlg::OnTtyPort)
    EVT_RADIOBOX(ID_TTY_BAUD,     SerialCfgDlg::OnTtyBaud)
    EVT_RADIOBOX(ID_TTY_PARITY,   SerialCfgDlg::OnTtyParity)
    EVT_RADIOBOX(ID_TTY_BITS,     SerialCfgDlg::OnTtyBits)
    EVT_RADIOBOX(ID_TTY_STOPBITS, SerialCfgDlg::OnTtyStopBits)
    EVT_RADIOBOX(ID_TTY_FLOWCTL,  SerialCfgDlg::OnTtyFlowCtl)
    EVT_BUTTON(wxID_ANY,          SerialCfgDlg::OnButton)
END_EVENT_TABLE()


SerialCfgDlg::SerialCfgDlg(wxWindow *parent, SerialCfgState *cfg) :
    wxDialog(parent, -1, wxT("Serial Port Configuration"),
             wxDefaultPosition, wxDefaultSize,
             wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER),
    hTtyEnable(NULL),
    hPort(NULL),
    hBaud(NULL),
    hParity(NULL),
    hBits(NULL),
    hStopbits(NULL),
    hFlowctl(NULL),
    hBtnRevert(NULL),
    hBtnOK(NULL),
    hBtnCancel(NULL),
    m_oldcfg(cfg),      // the existing configuration
    m_cfg   (*cfg)      // working copy we manipulate
{
    const int h_text_margin = 8;

    hTtyEnable = new wxCheckBox(this, ID_TTY_ENABLE,
                                      wxT("Associate serial port with terminal"));

    {
        wxString choices[] = { wxT("Com1"),
                               wxT("Com2"),
                               wxT("Com3"),
                               wxT("Com4") };
        hPort = new wxRadioBox(this, ID_TTY_PORT,
                               "Serial port",
                               wxDefaultPosition, wxDefaultSize,
                               4, choices,
                               1, wxRA_SPECIFY_ROWS);
    }

    {
        wxString choices[] = { wxT("110"),
                               wxT("300"),
                               wxT("1200"),
                               wxT("2400") };
        hBaud = new wxRadioBox(this, ID_TTY_BAUD,
                               "Baud Rate",
                               wxDefaultPosition, wxDefaultSize,
                               4, choices,
                               1, wxRA_SPECIFY_ROWS);
    }

    {
        wxString choices[] = { wxT("None"),
                               wxT("Odd"),
                               wxT("Even"),
                               wxT("Mark"),
                               wxT("Space") };
        hParity = new wxRadioBox(this, ID_TTY_PARITY,
                               "Parity",
                               wxDefaultPosition, wxDefaultSize,
                               5, choices,
                               1, wxRA_SPECIFY_ROWS);
    }

    {
        wxString choices[] = { wxT("7"),
                               wxT("8") };
        hBits = new wxRadioBox(this, ID_TTY_BITS,
                               "Bits/Byte",
                               wxDefaultPosition, wxDefaultSize,
                               2, choices,
                               1, wxRA_SPECIFY_ROWS);
    }

    {
        wxString choices[] = { wxT("1"),
                               wxT("2") };
        hStopbits = new wxRadioBox(this, ID_TTY_STOPBITS,
                               "Stop Bits",
                               wxDefaultPosition, wxDefaultSize,
                               2, choices,
                               1, wxRA_SPECIFY_ROWS);
    }

    {
        wxString choices[] = { wxT("none"),
                               wxT("rts/cts") };
        hFlowctl = new wxRadioBox(this, ID_TTY_FLOWCTL,
                               "Flow control",
                               wxDefaultPosition, wxDefaultSize,
                               2, choices,
                               1, wxRA_SPECIFY_ROWS);
    }

    wxBoxSizer *ctlsizer = new wxBoxSizer(wxVERTICAL);
    ctlsizer->Add(hTtyEnable, 0, wxALL, 5);
    ctlsizer->Add(hPort,      0, wxALL, 5);
    ctlsizer->Add(hBaud,      0, wxALL, 5);
    ctlsizer->Add(hParity,    0, wxALL, 5);
    ctlsizer->Add(hBits,      0, wxALL, 5);
    ctlsizer->Add(hStopbits,  0, wxALL, 5);
    ctlsizer->Add(hFlowctl,   0, wxALL, 5);

    // put three buttons side by side
    hBtnRevert = new wxButton(this, ID_BTN_REVERT, wxT("Revert"));
    hBtnOK     = new wxButton(this, wxID_OK,       wxT("OK"));
    hBtnCancel = new wxButton(this, wxID_CANCEL,   wxT("Cancel"));

    wxBoxSizer *button_sizer = new wxBoxSizer( wxHORIZONTAL );
    button_sizer->Add(hBtnRevert, 0, wxALL, 10);
    button_sizer->Add(hBtnOK,     0, wxALL, 10);
    button_sizer->Add(hBtnCancel, 0, wxALL, 10);
#ifdef __WXMAC__
    // the cancel button was running into the window resizing grip
    button_sizer->AddSpacer(10);
#endif
    hBtnRevert->Disable();      // until something changes

    // config grids on top, confirmation buttons on the bottom
    wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );
    topsizer->Add(ctlsizer,     1, wxEXPAND);           // vertically stretchable
    topsizer->Add(button_sizer, 0, wxALIGN_RIGHT);      // vertically unstretchable

    UpdateDlg();        // select current options

    // tell the thing to get to work
    SetSizer(topsizer);                 // use the sizer for layout
    topsizer->SetSizeHints(this);       // set size hints to honor minimum size

    GetDefaults();              // get default size & location
}


// update the display to reflect the current state
void
SerialCfgDlg::UpdateDlg()
{
    bool tty_enabled = m_cfg.getTtyEnable();
    hTtyEnable->SetValue(tty_enabled);

    int port = m_cfg.getPort();
    hPort->SetSelection(port-1);

    int baud = m_cfg.getBaudRate();
    int baud_choices[] = { 110, 300, 1200, 2400 };
    int idx = std::find(&baud_choices[0], &baud_choices[4], baud)
            - &baud_choices[0];
    hBaud->SetSelection(idx);

    int parity = m_cfg.getParity();
    int parity_choices[] = { ctb::ParityNone,
                             ctb::ParityOdd,
                             ctb::ParityEven,
                             ctb::ParityMark,
                             ctb::ParitySpace };
    idx = std::find(&parity_choices[0], &parity_choices[5], parity)
        - &parity_choices[0];
    hParity->SetSelection(idx);

    int bits = m_cfg.getBits();
    int bits_choices[] = { 7, 8 };
    idx = std::find(&bits_choices[0], &bits_choices[2], bits)
        - &bits_choices[0];
    hBits->SetSelection(idx);

    int stopbits = m_cfg.getStopBits();
    int stop_bits_choices[] = { 1, 2 };
    idx = std::find(&stop_bits_choices[0], &stop_bits_choices[2], stopbits)
        - &stop_bits_choices[0];
    hStopbits->SetSelection(idx);

    bool flow = m_cfg.getFlowctl();
    hFlowctl->SetSelection(flow ? 1 : 0);

    // if we don't want to associate a serial port,
    // disable all the controls
    hPort->Enable(tty_enabled);
    hBaud->Enable(tty_enabled);
    hParity->Enable(tty_enabled);
    hBits->Enable(tty_enabled);
    hStopbits->Enable(tty_enabled);
    hFlowctl->Enable(tty_enabled);

    // disable the "Revert" button if there haven't been any changes
    bool changed = (*m_oldcfg != m_cfg);
    hBtnRevert->Enable(changed);
}


void
SerialCfgDlg::OnTtyEnable( wxCommandEvent& WXUNUSED(event) )
{
    bool checked = hTtyEnable->IsChecked();
    m_cfg.setTtyEnable(checked);
    UpdateDlg();
}


void
SerialCfgDlg::OnTtyPort( wxCommandEvent& WXUNUSED(event) )
{
    m_cfg.setPort(hPort->GetSelection()+1);
    UpdateDlg();
}


void
SerialCfgDlg::OnTtyBaud( wxCommandEvent& WXUNUSED(event) )
{
    int baud_choices[] = { 110, 300, 1200, 2400 };
    m_cfg.setBaudRate(baud_choices[hBaud->GetSelection()]);
    UpdateDlg();
}


void
SerialCfgDlg::OnTtyParity( wxCommandEvent& WXUNUSED(event) )
{
    ctb::Parity parity_choices[] = {
            ctb::ParityNone,
            ctb::ParityOdd,
            ctb::ParityEven,
            ctb::ParityMark,
            ctb::ParitySpace };
    m_cfg.setParity(parity_choices[ hParity->GetSelection() ]);
    UpdateDlg();
}


void
SerialCfgDlg::OnTtyBits( wxCommandEvent& WXUNUSED(event) )
{
    int bits_choices[] = { 7, 8 };
    m_cfg.setBits(bits_choices[hBits->GetSelection()]);
    UpdateDlg();
}


void
SerialCfgDlg::OnTtyStopBits( wxCommandEvent& WXUNUSED(event) )
{
    int bits_choices[] = { 1, 2 };
    m_cfg.setStopBits(bits_choices[hStopbits->GetSelection()]);
    UpdateDlg();
}


void
SerialCfgDlg::OnTtyFlowCtl( wxCommandEvent& WXUNUSED(event) )
{
    bool rtscts = (hFlowctl->GetSelection() == 1);
    m_cfg.setFlowctl(rtscts);
    UpdateDlg();
}


// used for all dialog button presses
void
SerialCfgDlg::OnButton(wxCommandEvent& event)
{
    switch (event.GetId()) {

        case ID_BTN_REVERT:
            m_cfg = *m_oldcfg;
            UpdateDlg();
            break;

        case wxID_OK:
            SaveDefaults();         // save location & size of dlg
            *m_oldcfg = m_cfg;      // send latest state back to caller
            EndModal(EMU_OK);
            break;

        case wxID_CANCEL:
            SaveDefaults();         // save location & size of dlg
            EndModal(EMU_FAIL);
            break;

        default:
            assert(0);
            break;
    }
}


// save dialog options to the config file
void
SerialCfgDlg::SaveDefaults()
{
    // save position and size
    std::string subgroup("ui/serialcfgdlg");
    Host::ConfigWriteWinGeom(this, subgroup);
}


void
SerialCfgDlg::GetDefaults()
{
    // see if we've established a favored location and size
    std::string subgroup("ui/serialcfgdlg");
    Host::ConfigReadWinGeom(this, subgroup);
}
