#ifndef _INCLUDE_UI_SYSTEM_H_
#define _INCLUDE_UI_SYSTEM_H_

#include "wx/wx.h"
#include "wx/fileconf.h"        // required for wxFileConfig

#include "w3300.h"

// ==================== exported by UiSystem.cpp ====================

// these are menu IDs so the MyApp::OnHelpLauncher() konws
// what item is chosen.
enum {
        Help_Quickstart = 200,
//      Help_Configure,
//      Help_Keyboard,
//      Help_Menus,
//      Help_Printer,
//      Help_Script,
//      Help_DiskFactory,
//      Help_DiskCheat,
        Help_Website,
        Help_About = wxID_ABOUT,
    };

// a macro to cause the help menu to do something
#define HELP_MENU_EVENT_MAPPINGS() \
    EVT_MENU (Help_Website,          MyApp::OnHelp_Launcher) \
    EVT_MENU (Help_About,            MyApp::OnHelp_About)


// Define a new application type, each program should derive a class from wxApp
class MyApp : public wxApp
{
public:
    // override base class virtuals
    // ----------------------------

    // this one is called on application startup and is a good place for the app
    // initialization (doing it here and not in the ctor allows to have an error
    // return: if OnInit() returns false, the application terminates)
    virtual bool OnInit();

    // set the command line parsing options
    virtual void OnInitCmdLine(wxCmdLineParser& parser);

    // after the command line has been parsed, decode what it finds
    virtual bool OnCmdLineParsed(wxCmdLineParser& parser);

    // called whenever there is nothing else to do
    void OnIdle(wxIdleEvent &event);

    // This is called just before quitting the entire app,
    // but before wxWindows cleans up its internal resources.
    virtual int OnExit();

    // shut down the application
    static void Terminate();

    // indicate user wants to reconfigure the system
    static void Reconfigure();

    // Utility function that serves all others.
    // Ask user to provide a file location.
    static int FileReq(int requestor, wxString title, int readonly, wxString *fullpath);

    // create a help menu, used for all frames that care
    static wxMenu* MakeHelpMenu();
    void OnHelp_Launcher(wxCommandEvent& event);
    void OnHelp_About(wxCommandEvent& event);

    static wxFileConfig *m_config;      // configuration file object

private:
    static bool m_do_reconfig;          // when set, bring up config dialog

    // help ensure an orderly shutdown
    static enum term_state_t { RUNNING, TERMINATING, TERMINATED } m_term_state;
    static void SetTermState(term_state_t newstate) { m_term_state = newstate; }
    static term_state_t GetTermState() { return m_term_state; }

    // initialize "static" class members
    static void InitStatics();

    DECLARE_EVENT_TABLE()
};


// =============================================================
// exported by UI
// =============================================================

// inform the UI how far along the simulation is in emulated time
void UiSetSimSeconds(unsigned long seconds, float relative_speed);

// send an error/warning to the user that requires an action to close
void UiError(const char *fmt, ...);
void UiWarn(const char *fmt, ...);
void UiInfo(const char *fmt, ...);

// get a YES/NO confirmation
bool UiConfirm(const char *fmt, ...);

// called periodically by the core emulation so UI events can occur
void UiYield(void);

#endif // _INCLUDE_UI_SYSTEM_H_
