#ifndef _UI_SYSTEM_CONFIG_DLG_H_
#define _UI_SYSTEM_CONFIG_DLG_H_

// forward declaration
class SystemCfg;
class SerialCfgDlg;

// ----------------------------------------------------------------------------
// SystemCfgDlg class definition
// ----------------------------------------------------------------------------

// modal dialog box for configuring the emulated computer
// the return code from the invoking ShowModal() call will
// be one of the values from the return_status_t enum.
class SystemCfgDlg : public wxDialog
{
public:
    SystemCfgDlg(wxFrame *parent, SystemCfg *cfg);
    // ~SystemCfgDlg();

    enum return_status_t { CANCELLED, UNCHANGED, CHANGED };

private:
    SystemCfg *m_oldcfg;                // configuration object
    SystemCfg *m_config;                // working copy

    wxCheckBox *hSpeedRegulated;        // whether the CPU emulation is realtime
    wxChoice   *hMemsize;               // main memory size
    wxChoice   *hNumTerms;              // number of terminals
    wxChoice   *hNumCassettes;          // number of cassette drives
    wxButton   *hSerialCfg[MAX_TERMINALS]; // serial terminalal configuration
#if 0
    wxCheckBox *hWarnIO;        // whether to want on access to non-existing IO devices
                                // the real Wang hardware ignored such actions
                                // the emulator has support to catch such actions, which is
                                // useful for debugging
#endif
    wxButton   *hBtnRevert;
    wxButton   *hBtnOK;
    wxButton   *hBtnCancel;

    // helper routines
    void UpdateDlg();
    void RefitDlg();

    // save/get dialog box geometry the ini file
    void SaveDefaults();
    void GetDefaults();

    // ---- event handlers ----
    void OnMemsizeChoice( wxCommandEvent& WXUNUSED(event) );
    void OnSpeedRegulated( wxCommandEvent& WXUNUSED(event) );
    void OnNumTerminals( wxCommandEvent& WXUNUSED(event) );
    void OnNumCassetteDrives( wxCommandEvent& WXUNUSED(event) );
    void OnSerialCfg(wxCommandEvent& event);
    void OnButton(wxCommandEvent& event);
#if 0
    void OnWarnIO( wxCommandEvent& WXUNUSED(event) );
#endif

    CANT_ASSIGN_OR_COPY_CLASS(SystemCfgDlg);
    DECLARE_EVENT_TABLE()
};

#endif // _UI_SYSTEM_CONFIG_DLG_H_
