#ifndef _INCLUDE_W3300_H_
#define _INCLUDE_W3300_H_

#include <cassert>

// some simple type definitions

#ifdef __GNUC__
    typedef unsigned long long uint64;
    typedef   signed long long  int64;
#else
    typedef unsigned __int64 uint64;
    typedef   signed __int64  int64;
#endif
typedef unsigned long  uint32;
typedef   signed long   int32;
typedef unsigned short uint16;
typedef   signed short  int16;
typedef unsigned char  uint8;
typedef   signed char   int8;

// generic status values
#define EMU_OK   0
#define EMU_FAIL 1

// maximum number of terminals supported by emulator
#define MAX_TERMINALS 8

// used for in Term3315 and Term3315_pun indexing two tape units
enum unit_t { rdr_unit, pun_unit, end_unit };

// used to signal that something has changed in the tape interface
// the bitmap may be OR'd to signal multiple changes
enum punch_event_t {
    pe_mount    = 0x0001,  // tape was mounted or unmounted
    pe_position = 0x0002,  // tape position changed
    pe_punchen  = 0x0004,  // punch enable state changed
    pe_all      = 0x0007   // indicate everything has changed
    };

// Put this at the end of a class declaration to mae the class
// uncopyable and unassignable

#define CANT_ASSIGN_CLASS(ClassName)            \
  private:                                      \
    ClassName& operator=(const ClassName& rhs);

#define CANT_ASSIGN_OR_COPY_CLASS(ClassName)    \
  private:                                      \
    ClassName& operator=(const ClassName& rhs); \
    ClassName(ClassName& obj)


// =============================================================
// host specific defines
// =============================================================

#if defined(__WXMSW__)
    #include <windows.h>        // _MAX_PATH and GetFullPathName()
    #define MAX_PATHLEN (_MAX_PATH)
#elif defined(PATH_MAX)
    #define MAX_PATHLEN (PATH_MAX)
#else
    #define MAX_PATHLEN (512)
#endif

#endif // _INCLUDE_W3300_H_
